"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresSerializer = void 0;
const builder_1 = require("@sqb/builder");
const reservedWords = ['comment'];
class PostgresSerializer {
    constructor() {
        this.dialect = 'postgres';
    }
    isReservedWord(ctx, s) {
        return s && typeof s === 'string' &&
            reservedWords.includes(s.toLowerCase());
    }
    serialize(ctx, type, o, defFn) {
        switch (type) {
            case builder_1.SerializationType.SELECT_QUERY:
                return this._serializeSelect(ctx, o, defFn);
            case builder_1.SerializationType.COMPARISON_EXPRESSION:
                return this._serializeComparison(ctx, o, defFn);
            case builder_1.SerializationType.EXTERNAL_PARAMETER:
                return this._serializeParameter(ctx, o);
        }
    }
    _serializeSelect(ctx, o, defFn) {
        let out = defFn(ctx, o);
        const limit = o.limit || 0;
        const offset = Math.max((o.offset || 0), 0);
        if (limit)
            out += '\nLIMIT ' + limit;
        if (offset)
            out += (!limit ? '\n' : ' ') + 'OFFSET ' + offset;
        return out;
    }
    _serializeComparison(ctx, o, defFn) {
        if (typeof o.right === 'string') {
            if (o.right.startsWith('(')) {
                if (o.operatorType === 'eq')
                    o.symbol = 'in';
                if (o.operatorType === 'ne')
                    o.symbol = 'not in';
            }
            else {
                if (o.right.substring(0, 1) === '$') {
                    if (o.operatorType === 'in') {
                        o.symbol = '=';
                        o.right = 'ANY(' + o.right + ')';
                    }
                    if (o.operatorType === 'notIn') {
                        o.symbol = '!=';
                        o.right = 'ANY(' + o.right + ')';
                    }
                }
            }
        }
        return defFn(ctx, o);
    }
    _serializeParameter(ctx, o) {
        let prmValue = ctx.params && ctx.params[o.name];
        if (o.isArray && !Array.isArray(prmValue))
            prmValue = [prmValue];
        ctx.queryParams = ctx.queryParams || [];
        ctx.queryParams.push(prmValue);
        return '$' + ctx.queryParams.length;
    }
}
exports.PostgresSerializer = PostgresSerializer;
